#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup


setup(name='aiologfields',
      version='2017.12.2',
      description='aiologfields: inject Task-context-fields into loggers',
      author='Caleb Hattingh',
      author_email='caleb.hattingh@gmail.com',
      url='https://github.com/cjrh/aiologfields',
      py_modules=['aiologfields'],
     )
